@@ -1,17 +1,17 @@
-from __future__ import annotations
+#from __future__ import annotations
 
 import functools
 import itertools
 
 from dataclasses import dataclass, field
-from typing import Callable, Iterator, Optional, TypeVar
+from typing import Callable, Dict, List, Iterator, Optional, Set, TypeVar
 
 from .data import PCP, PCPRule  
 
 #region Helper methods
 
 from math import factorial as _math_factorial
-_factorial = functools.lru_cache(_math_factorial)
+_factorial = functools.lru_cache()(_math_factorial)
 
 T = TypeVar('T')
 def _apply(f: Callable[[T,T],T], lhs: Optional[T], rhs: Optional[T]) -> Optional[T]:
@@ -47,22 +47,22 @@ class CompositionGenerator:
   ``max_allowed`` is them maximum number of characters allowed for each charset.
   """
   length: int
-  required: list[int]
-  max_allowed: list[Optional[int]]
+  required: List[int]
+  max_allowed: List[Optional[int]]
   
-  def compositions(self, preferences: list[int] = []) -> Iterator[Composition]:
+  def compositions(self, preferences: List[int] = []) -> Iterator['Composition']:
     """
     Yields the compositions for this generator.
 
     Args:
-        preferences (list[int], optional): List of charsets to prefer when generating compositions. Defaults to [].
+        preferences (List[int], optional): List of charsets to prefer when generating compositions. Defaults to [].
 
     Yields:
         Iterator[PasswordComposition]: Compositions of the given length.
     """
     required_length = sum(self.required)
     remaining_length = self.length - required_length
-    remaining_characters: list[list[int]] = []
+    remaining_characters: List[List[int]] = []
     
     # Compute which characters can still appear in the password
     for i in range(remaining_length):
@@ -107,13 +107,13 @@ class CompositionRestrictions:
   ``charsets_prohibited_locations`` is the list of prohibited_locations characters for each charset.
   """
   max_consecutive: Optional[int] = None
-  prohibited_substrings: Optional[set[prohibitedSubstring]] = None
-  charsets_max_consecutive: Optional[list[Optional[int]]] = None
-  charsets_required_locations: Optional[list[Optional[set[int]]]] = None
-  charsets_prohibited_locations: Optional[list[Optional[set[int]]]] = None
+  prohibited_substrings: Optional[Set['ProhibitedSubstring']] = None
+  charsets_max_consecutive: Optional[List[Optional[int]]] = None
+  charsets_required_locations: Optional[List[Optional[Set[int]]]] = None
+  charsets_prohibited_locations: Optional[List[Optional[Set[int]]]] = None
 
   @staticmethod
-  def merge(lhs: CompositionRestrictions, rhs: CompositionRestrictions) -> CompositionRestrictions:
+  def merge(lhs: 'CompositionRestrictions', rhs: 'CompositionRestrictions') -> 'CompositionRestrictions':
     """
     Merge two CompositionRestrictions objects.
     
@@ -182,12 +182,12 @@ class CompositionRestrictions:
     
 
 @dataclass(frozen=True)
-class prohibitedSubstring:
+class ProhibitedSubstring:
   """
   Represents a prohibited substring. Keeps track of both the actual string (needed for de-duplication) and the charset composition of that string.
   """
   substring: str
-  composition: list[int] = field(compare=False, hash=False)
+  composition: List[int] = field(compare=False, hash=False)
   
   
 #endregion
@@ -201,19 +201,19 @@ class Composition():
 
   ``composition`` is a list of how many characters from each charset make up this composition.
   """
-  composition: list[int]
+  composition: List[int]
   
   def get_search_space(self, 
-                       restrictions_set: set[CompositionRestrictions], 
-                       charset_sizes: list[int]) -> int:
+                       restrictions_set: Set[CompositionRestrictions], 
+                       charset_sizes: List[int]) -> int:
     """
     Get the search space for this composition.
 
     Return value will be exact for composition without restrictions, compositions with a single restriction, and for some simple combination of restrictions. Otherwise, the resulting search space will be smaller than it should be as restrictions will be remoed multiple times. The only way to solve this is to expand combinations and/or permutations, but this is not tractable.
 
     Args:
-        restrictions_set (set[CompositionRestrictions]): Set of restrictions for valid passwords.
-        charset_sizes (list[int]): How many characters are in each charset of the composition.
+        restrictions_set (Set[CompositionRestrictions]): Set of restrictions for valid passwords.
+        charset_sizes (List[int]): How many characters are in each charset of the composition.
 
     Returns:
         int: Estimate of the search space for this composition. 
@@ -222,8 +222,8 @@ class Composition():
   
   
   def _get_search_space(self, 
-                       restrictions_set: set[CompositionRestrictions], 
-                       charset_sizes: list[int]) -> int: 
+                       restrictions_set: Set[CompositionRestrictions], 
+                       charset_sizes: List[int]) -> int: 
     """"""
     # If no restrictions, just return the  basic search space calculation
     if restrictions_set is None or len(restrictions_set) == 0:
@@ -328,11 +328,11 @@ class Composition():
     return search_space
   
   
-  def _get_simple_search_space(self, charset_sizes: list[int]) -> int:
+  def _get_simple_search_space(self, charset_sizes: List[int]) -> int:
     """Get the search space for the given composition. Does not consider composition restrictions.
 
     Args:
-        charset_sizes (list[int]): How many characters make up each charset.
+        charset_sizes (List[int]): How many characters make up each charset.
 
     Returns:
         int: The search space for the given composition.
@@ -373,12 +373,12 @@ def get_machine_strength(pcp: PCP) -> int:
   return _get_strength(pcp)
 
   
-def get_human_strength(pcp: PCP, charset_preferences: list[str] = DEFAULT_CHARSET_PREFERENCES) -> int:
+def get_human_strength(pcp: PCP, charset_preferences: List[str] = DEFAULT_CHARSET_PREFERENCES) -> int:
   """Get the strength of human-generated passwords for the given the policy. Assumes (1) users pick the shortest possible password, (2) users pick their preferred charsets when available, and (3) on average the adversary will guess the password after guessing half the passwords.
 
   Args:
       pcp (PCP): Policy for which to measure strength.
-      charset_preferences (list[str], optional): Which charsets the user prefers to use if available, in order. Defaults to DEFAULT_CHARSET_PREFERENCES.
+      charset_preferences (List[str], optional): Which charsets the user prefers to use if available, in order. Defaults to DEFAULT_CHARSET_PREFERENCES.
 
   Returns:
       int: The average number of guesses needed to find a password created for the given policy.
@@ -386,12 +386,12 @@ def get_human_strength(pcp: PCP, charset_preferences: list[str] = DEFAULT_CHARSE
   return _get_strength(pcp, charset_preferences)
 
 
-def _get_strength(pcp: PCP, charset_preferences: list[str] = []) -> int:
+def _get_strength(pcp: PCP, charset_preferences: List[str] = []) -> int:
   """Get the strength of the given policy. Assumes (1) the shortest possible password will be used, (2) charsets will be used based on the given preference (if any), and (3) on average the adversary will guess the password after guessing half the passwords.
 
   Args:
       pcp (PCP): Policy for which to measure strength.
-      charset_preferences (list[str], optional): Which charsets are preferred to use if available, in order. Defaults to DEFAULT_CHARSET_PREFERENCES.
+      charset_preferences (List[str], optional): Which charsets are preferred to use if available, in order. Defaults to DEFAULT_CHARSET_PREFERENCES.
 
   Returns:
       int: The average number of guesses needed to find a password created for the given policy.
@@ -406,7 +406,7 @@ def _get_strength(pcp: PCP, charset_preferences: list[str] = []) -> int:
   preferences = [charset_mapping[preference] for preference in charset_preferences if preference in charset_mapping]
   
   # Get the set of password compositions produced by the policy. Also keep track of restrictions for each composition.
-  compositions: dict[Composition, set[CompositionRestrictions]] = dict()
+  compositions: Dict[Composition, Set[CompositionRestrictions]] = dict()
   for generator, restrictions_set in get_generators(pcp, charset_mapping).items():
     for composition in generator.compositions(preferences):
       if not composition in compositions:
@@ -423,23 +423,23 @@ def _get_strength(pcp: PCP, charset_preferences: list[str] = []) -> int:
   return search_space // 2
 
 
-def get_generators(pcp: PCP, charset_mapping: dict[str,int]) -> dict[CompositionGenerator,set[CompositionRestrictions]]:
+def get_generators(pcp: PCP, charset_mapping: Dict[str,int]) -> Dict[CompositionGenerator,Set[CompositionRestrictions]]:
   """[summary]
 
   Args:
       pcp (PCP): Policy for which to measure strength.
-      charset_mapping (dict[str,int]): Mapping from charsets to indices.
+      charset_mapping (Dict[str,int]): Mapping from charsets to indices.
 
   Returns:
-      dict[CompositionGenerator, set[CompositionRestrictions]]: Generators with associated restrictions.
+      Dict[CompositionGenerator, Set[CompositionRestrictions]]: Generators with associated restrictions.
   """
   smallest_min_length = min([r.min_length for r in pcp.rules])
-  generators: dict[CompositionGenerator, set[CompositionRestrictions]] = dict()
+  generators: Dict[CompositionGenerator, Set[CompositionRestrictions]] = dict()
   
   for rule in filter(lambda r: r.min_length == smallest_min_length, pcp.rules):
     # Get PasswordCompositionGenerator relevant data from the rule 
-    required: list[int] = [0] * len(charset_mapping)
-    max_allowed: list[Optional[int]] = [None] * len(charset_mapping)
+    required: List[int] = [0] * len(charset_mapping)
+    max_allowed: List[Optional[int]] = [None] * len(charset_mapping)
     
     try:
       restriction = _get_composition_restrictions(rule, pcp.charsets, charset_mapping)
@@ -483,21 +483,21 @@ def get_generators(pcp: PCP, charset_mapping: dict[str,int]) -> dict[Composition
 
 def _get_composition_restrictions(
   rule: PCPRule,
-  charsets: dict[str,str],
-  charset_mapping: dict[str,int]) -> Optional[CompositionRestrictions]:
+  charsets: Dict[str,str],
+  charset_mapping: Dict[str,int]) -> Optional[CompositionRestrictions]:
   """
   Get a composition restriction object, if applicable, for the given rule.
 
   Args:
       rule (PCPRule): Rule to get restrictions from.
-      charsets (dict[str,str]): Charsets defined by the policy.
-      charset_mapping (dict[str,int]): Mapping from charsets to indices.
+      charsets (Dict[str,str]): Charsets defined by the policy.
+      charset_mapping (Dict[str,int]): Mapping from charsets to indices.
 
   Returns:
       Optional[CompositionRestrictions]: Restrictions, or ``None`` if there are none.
   """
   # Process prohibited substrings
-  prohibited_substrings: Optional[set[prohibitedSubstring]] = None
+  prohibited_substrings: Optional[Set[ProhibitedSubstring]] = None
   
   if rule.prohibited_substrings is not None:
     prohibited_substrings = set()
@@ -509,12 +509,12 @@ def _get_composition_restrictions(
           if char in charset:
             composition[charset_mapping[name]] += 1
             break
-      prohibited_substrings.add(prohibitedSubstring(substring, composition))
+      prohibited_substrings.add(ProhibitedSubstring(substring, composition))
   
   # Process charset_* restrictions
-  charsets_max_consecutive: Optional[list[Optional[int]]] = None
-  charsets_required_locations: Optional[list[Optional[set[int]]]] = None
-  charsets_prohibited_locations: Optional[list[Optional[set[int]]]] = None
+  charsets_max_consecutive: Optional[List[Optional[int]]] = None
+  charsets_required_locations: Optional[List[Optional[Set[int]]]] = None
+  charsets_prohibited_locations: Optional[List[Optional[Set[int]]]] = None
   
   if rule.charset_requirements is not None:    
     charsets_max_consecutive = [None] * len(charset_mapping)
