@@ -84,6 +84,24 @@ ALPHABET_CHARSETS: Dict[str,str] = {
   "symbols": punctuation  
 }
 
+def SimplePCP(*args, **kwargs) -> 'PCP':
+  """Creates a PCP object with a single PCPRule with the given arguments. 
+  
+  ``min_length`` is the minimum number of characters in the password. Must be 1 or higher.
+  ``max_length`` is the maximum number of characters in the password.
+  ``max_consecutive`` is the maximum number of consecutive identical characters allowed.
+  ``prohibited_substrings`` is a list of substrings not allowed in the password.
+  
+  ``require`` is the list of charsets that must appear in the password.
+  ``require_subset`` is a list of charsets for which sum number must appear in the password.
+  
+  ``charset_requirements`` is the set of additional requirements for each charset.
+  
+  Returns:
+      PCP: PCP object with a single PCPRule created using the given arguments.
+  """
+  return PCP([PCPRule(*args,**kwargs)])
+
 @dataclass()
 class PCP():
   """
@@ -96,19 +114,20 @@ class PCP():
   rules: List['PCPRule']
   charsets: Dict[str,str] = field(default_factory=DEFAULT_CHARSETS.copy)
   
-  def dumps(self) -> str:
+  def dumps(self, **kwargs) -> str:
     """Create a json representation of this object.
 
     Returns:
         str: Representation of this object in json.
+        kwargs: keyword args to pass to json.dumps.
     """
     
     # Simplified outputs if features not used
     if self.charsets == DEFAULT_CHARSETS or self.charsets == ALPHABET_CHARSETS:
       if len(self.rules) == 1:
-        return json.dumps(_to_json_compatible_object(self.rules[0]))
+        return json.dumps(_to_json_compatible_object(self.rules[0]), **kwargs)
       else:
-        return json.dumps({'rules': [_to_json_compatible_object(rule) for rule in self.rules]})
+        return json.dumps({'rules': [_to_json_compatible_object(rule) for rule in self.rules]}, **kwargs)
 
     # Get a diff between the defined charset and the default charset. Only dump this diff.
     charsets_diff: Dict[str,str] = {}
@@ -122,7 +141,7 @@ class PCP():
       if name not in self.charsets:
         charsets_diff[name] = None
     
-    return json.dumps({'charsets': charsets_diff, 'rules': [_to_json_compatible_object(rule) for rule in self.rules]})
+    return json.dumps({'charsets': charsets_diff, 'rules': [_to_json_compatible_object(rule) for rule in self.rules]}, **kwargs)
     
     
   @staticmethod
@@ -198,7 +217,7 @@ class PCPRule():
   
   ``charset_requirements`` is the set of additional requirements for each charset.
   """
-  min_length: int = 1
+  min_length: Optional[int] = 1
   max_length: Optional[int] = None
   max_consecutive: Optional[int] = None
   prohibited_substrings: Optional[Set[str]] = None
